/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.einstellungen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;

import com.bitctrl.lib.eclipse.resources.WidgetColorRegistry;

import de.bsvrz.buv.plugin.anlagenstatus.internal.AnlagenStatusKnotenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.views.AnlagenStatus;
import de.bsvrz.buv.rw.bitctrl.eclipse.widgets.ColorPicker;

/**
 * Preference Page f&uuml;r das Plug-in Anlagenstatus.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public class AnlagenStatusPreferencePage extends PreferencePage implements
		IWorkbenchPreferencePage {

	/** Die Context-Hilfe-ID. */
	private static final String HILFE_ID = PluginAnlagenStatus.PLUGIN_ID + "."
			+ AnlagenStatusPreferencePage.class.getSimpleName();

	/** Workbench. */
	private IWorkbench workbench;

	/**
	 * Alle Darstellungen.
	 */
	private final Map<AnlagenStatusKnotenStatus, Long> farbenMap = new HashMap<AnlagenStatusKnotenStatus, Long>();

	/**
	 * Die Images der Page.
	 */
	private final List<Image> imagelist = new ArrayList<Image>();

	/**
	 * Ver&auml;ndern: Farben benutzen.
	 */
	private Button steuerModuleInitialAusblenden;

	/** ResourceBundle. */
	private final ResourceBundle resourceBundle;

	/** Einstellung Linienst&auml;rke Verbindungslinien. */
	private Spinner linienDicke;

	/** Einstellung Farbe der Verbindungslinien. */
	private ColorPicker linienFarbe;

	/** Einstellung Rahmenbreite. */
	private Spinner rahmenBreite;

	/** Einstellung Hintergrundfarbe der Anlagenteile. */
	private ColorPicker knotenFarbe;

	/** Viewer f&uuml;r Statusfarben. */
	private TableViewer farbenViewer;

	/** Button zum Bearbeiten der Statusfarben. */
	private Button statusFarbeBearbeitenButton;

	/** aktuell selektierter Status. */
	private AnlagenStatusKnotenStatus selectedStatus;

	/** Button zur Auswahl der initialen Hierarchierichtung. */
	private Button hierarchieInitial;

	/**
	 * Konstruktor.
	 */
	public AnlagenStatusPreferencePage() {
		resourceBundle = PluginAnlagenStatus.getDefault().getResourceBundle();

		setDescription(resourceBundle.getString("Einstellungen.description"));

		initMap();
	}

	/**
	 * Initialisiert die Farben-Map.
	 */
	private void initMap() {
		farbenMap.clear();
		final AnlagenStatusEinstellungen einstellungen = PluginEinstellungen
				.getInstance().getEinstellungen();

		for (AnlagenStatusKnotenStatus stat : AnlagenStatusKnotenStatus
				.values()) {
			farbenMap.put(stat, einstellungen.getStatusFarben().get(stat));
		}
	}

	/** {@inheritDoc} */
	@Override
	public void init(final IWorkbench w) {
		workbench = w;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected Control createContents(final Composite parent) {
		PlatformUI.getWorkbench().getHelpSystem().setHelp(parent,
				AnlagenStatusPreferencePage.HILFE_ID);

		final Composite composite = new Composite(parent, SWT.NONE);
		composite.setLayout(new GridLayout(1, false));
		composite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL
				| GridData.GRAB_HORIZONTAL | GridData.FILL_VERTICAL
				| GridData.GRAB_VERTICAL));
		// composite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL
		// | GridData.GRAB_HORIZONTAL | GridData.FILL_VERTICAL
		// | GridData.GRAB_VERTICAL));

		new Label(composite, SWT.NONE);
		steuerModuleInitialAusblenden = new Button(composite, SWT.CHECK);
		steuerModuleInitialAusblenden.setSelection(PluginEinstellungen
				.getInstance().getEinstellungen().isSteuerModuleAusblenden());
		steuerModuleInitialAusblenden.setText(resourceBundle
				.getString("EinstellungenSteuerModule.label"));
		steuerModuleInitialAusblenden
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						// update();
					}

				});

		hierarchieInitial = new Button(composite, SWT.CHECK);
		hierarchieInitial.setSelection(!PluginEinstellungen.getInstance()
				.getEinstellungen().isTopDown());
		hierarchieInitial.setText(resourceBundle
				.getString("EinstellungenHierarchie.label"));
		hierarchieInitial.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(final SelectionEvent e) {
				// update();
			}

		});
		// Dummy-Label fr Lcke
		new Label(composite, SWT.NONE);

		// Verbindungslinien
		final Group groupLinien = new Group(composite, SWT.NONE);
		groupLinien.setLayout(new GridLayout(5, false));
		// GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.BEGINNING)
		// .applyTo(groupLinien);
		groupLinien.setText("Verbindungslinien");

		Label label = new Label(groupLinien, SWT.NONE);
		label.setText("Linienbreite:");
		// label.setLayoutData(new GridData(GridData.FILL_HORIZONTAL
		// | GridData.HORIZONTAL_ALIGN_BEGINNING));
		GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER)
				.applyTo(label);

		linienDicke = new Spinner(groupLinien, SWT.BORDER);
		// linienDicke.setLayoutData(new GridData(GridData.FILL_HORIZONTAL
		// | GridData.HORIZONTAL_ALIGN_BEGINNING));
		GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER)
				.applyTo(linienDicke);
		linienDicke.setMinimum(1);
		linienDicke.setMaximum(10);
		linienDicke.setSelection(PluginEinstellungen.getInstance()
				.getEinstellungen().getLinienBreite());

		final GridData square = new GridData();
		square.grabExcessHorizontalSpace = true;
		square.grabExcessVerticalSpace = true;
		square.minimumHeight = 30;
		square.minimumWidth = 30;
		square.verticalAlignment = SWT.CENTER;

		@SuppressWarnings("unused")
		Label dummy = new Label(groupLinien, SWT.NONE);
		// GridDataFactory.fillDefaults().grab(true, false).applyTo(dummy);

		Label l1 = new Label(groupLinien, SWT.NONE);
		GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER)
				.applyTo(l1);
		l1.setText("Linienfarbe:");
		linienFarbe = new ColorPicker(groupLinien, SWT.FLAT);
		linienFarbe.setLayoutData(square);
		linienFarbe.setColor(ColorUtilities.longToColor(
				PluginEinstellungen.getInstance().getEinstellungen()
						.getLinienFarbe()).getRGB());
		linienFarbe
				.addSelectionChangedListener(new ISelectionChangedListener() {

					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {
						// final ISelection selection = event.getSelection();
						// final RGB rgb = (RGB) ((StructuredSelection)
						// selection)
						// .getFirstElement();
						// System.out.println("Farbe: " + rgb.toString());
						// // getDarstellungMap().get(currentSelection)
						// // .setVordergrundfarbe(
						// // ColorUtilities.rgbToLong(rgb));
						// // updateWidget();
					}

				});
		GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER)
				.applyTo(linienFarbe);

		// Label dummy = new Label(groupLinien, SWT.NONE);
		// GridDataFactory.fillDefaults().grab(true, false).applyTo(dummy);

		// Knoten
		final Group groupKnoten = new Group(composite, SWT.NONE);
		groupKnoten.setLayout(new GridLayout(5, false));
		// GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER)
		// .applyTo(groupKnoten);
		// groupLinien.setLayoutData(new GridData(GridData.FILL_HORIZONTAL
		// | GridData.GRAB_HORIZONTAL));
		groupKnoten.setText("Anlagenteile");

		label = new Label(groupKnoten, SWT.NONE);
		label.setText("Rahmenbreite:");
		GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER)
				.applyTo(label);

		rahmenBreite = new Spinner(groupKnoten, SWT.BORDER);
		rahmenBreite.setMinimum(1);
		rahmenBreite.setMaximum(10);
		rahmenBreite.setSelection(PluginEinstellungen.getInstance()
				.getEinstellungen().getRahmenBreite());
		GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER)
				.applyTo(rahmenBreite);

		// dummy
		dummy = new Label(groupKnoten, SWT.NONE);

		l1 = new Label(groupKnoten, SWT.NONE);
		GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER)
				.applyTo(l1);
		l1.setText("Hintergrundfarbe:");

		knotenFarbe = new ColorPicker(groupKnoten, SWT.FLAT);
		knotenFarbe.setLayoutData(square);
		knotenFarbe.setColor(ColorUtilities.longToColor(
				PluginEinstellungen.getInstance().getEinstellungen()
						.getKnotenBackGroundColor()).getRGB());
		knotenFarbe
				.addSelectionChangedListener(new ISelectionChangedListener() {

					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {
						// final ISelection selection = event.getSelection();
						// final RGB rgb = (RGB) ((StructuredSelection)
						// selection)
						// .getFirstElement();
						// // getDarstellungMap().get(currentSelection)
						// // .setVordergrundfarbe(
						// // ColorUtilities.rgbToLong(rgb));
						// // updateWidget();
					}

				});
		GridDataFactory.fillDefaults().align(SWT.BEGINNING, SWT.CENTER)
				.applyTo(knotenFarbe);

		// Dummy-Label fr Lcke
		// new Label(composite, SWT.NONE);

		// Statusfarben
		final Group groupStatusfarben = new Group(composite, SWT.NONE);
		groupStatusfarben.setLayout(new GridLayout(2, false));
		groupStatusfarben.setText("Zustandsfarben");
		farbenViewer = new TableViewer(groupStatusfarben);
		farbenViewer.getControl().setLayoutData(
				new GridData(GridData.FILL_HORIZONTAL
						| GridData.HORIZONTAL_ALIGN_FILL));
		// tv.getTree().setLayout(new GridLayout());
		// tv.getTree().setLayoutData(new GridData(GridData.FILL_BOTH));
		farbenViewer.setContentProvider(new ArrayContentProvider());
		farbenViewer.setLabelProvider(new StatusFarbenLabelProvider());
		farbenViewer.setInput(AnlagenStatusKnotenStatus.values());

		statusFarbeBearbeitenButton = new Button(groupStatusfarben, SWT.PUSH);
		statusFarbeBearbeitenButton.setText("Bearbeiten");
		statusFarbeBearbeitenButton
				.addSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(final SelectionEvent e) {
						final Point position = statusFarbeBearbeitenButton
								.getDisplay().map(statusFarbeBearbeitenButton,
										null, 0, 0);
						final Shell shell = new Shell(getShell());
						position.y = Math.max(0, position.y - 319);
						position.x = Math.max(0, position.x - 20);
						shell.setLocation(position);
						// shell.setSize(300, 300);
						final ColorDialog cd = new ColorDialog(shell);
						cd.setText("Farbe '" + selectedStatus.getName() + "'");
						cd.setRGB(ColorUtilities.longToColor(
								PluginEinstellungen.getInstance()
										.getEinstellungen()
										.getKnotenStatusBorderColor(
												selectedStatus)).getRGB());
						final RGB newColor = cd.open();
						if (newColor == null) {
							return;
						}

						farbenMap.put(selectedStatus, ColorUtilities
								.rgbToLong(newColor));
						farbenViewer.refresh();
					}
				});
		statusFarbeBearbeitenButton.setEnabled(false);

		farbenViewer
				.addSelectionChangedListener(new ISelectionChangedListener() {

					@Override
					public void selectionChanged(
							final SelectionChangedEvent event) {
						final ISelection selection = event.getSelection();
						if (!selection.isEmpty()) {
							final Object selected = ((StructuredSelection) selection)
									.getFirstElement();
							if (selected instanceof AnlagenStatusKnotenStatus) {
								selectedStatus = (AnlagenStatusKnotenStatus) selected;
							}
							statusFarbeBearbeitenButton.setEnabled(true);
						}
					}
				});

		checkPage();
		return composite;
	}

	/** {@inheritDoc} */
	@Override
	protected void performDefaults() {
		steuerModuleInitialAusblenden
				.setSelection(PluginStandardEinstellungen.STEUERMODULE_AUSBLENDEN_STANDARD);
		hierarchieInitial
				.setSelection(!PluginStandardEinstellungen.TOPDOWN_STANDARD);
		linienDicke
				.setSelection(PluginStandardEinstellungen.VERBINDUNGEN_LINIENBREITE_STANDARD);
		linienFarbe.setColor(ColorUtilities.longToColor(
				PluginStandardEinstellungen.VERBINDUNGEN_LINIENFARBE_STANDARD)
				.getRGB());
		knotenFarbe.setColor(ColorUtilities.longToColor(
				PluginStandardEinstellungen.KNOTEN_BACKGROUND_COLOR_STANDARD)
				.getRGB());
		rahmenBreite
				.setSelection(PluginStandardEinstellungen.KNOTEN_RAHMENBREITE_STANDARD);
		farbenMap.clear();
		farbenMap
				.put(
						AnlagenStatusKnotenStatus.UNBEKANNT,
						PluginStandardEinstellungen.KNOTEN_RAHMENFARBE_STATUS_UNBEKANNT);
		farbenMap
				.put(
						AnlagenStatusKnotenStatus.TEILGESTOERT,
						PluginStandardEinstellungen.KNOTEN_RAHMENFARBE_STATUS_TEILGESTOERT);
		farbenMap.put(AnlagenStatusKnotenStatus.GESTOERT,
				PluginStandardEinstellungen.KNOTEN_RAHMENFARBE_STATUS_GESTOERT);
		farbenMap.put(AnlagenStatusKnotenStatus.OK,
				PluginStandardEinstellungen.KNOTEN_RAHMENFARBE_STATUS_OK);
		statusFarbeBearbeitenButton.setEnabled(false);
		farbenViewer.getTable().deselectAll();
		farbenViewer.refresh();
	}

	/** {@inheritDoc} */
	@Override
	public boolean performOk() {
		performApply();
		return super.performOk();
	}

	/** {@inheritDoc} */
	@Override
	public void performApply() {
		final AnlagenStatusEinstellungen einstellungen = PluginEinstellungen
				.getInstance().getEinstellungen();

		if (isChanged()) {
			einstellungen
					.setSteuerModuleAusblenden(steuerModuleInitialAusblenden
							.getSelection());
			einstellungen.setTopDown(!hierarchieInitial.getSelection());
			RGB rgb = linienFarbe.getColorValue();
			einstellungen.setLinienFarbe(ColorUtilities.rgbToLong(rgb));
			einstellungen.setLinienBreite(linienDicke.getSelection());

			rgb = knotenFarbe.getColorValue();
			einstellungen.setKnotenBackGroundColor(ColorUtilities
					.rgbToLong(rgb));
			einstellungen.setRahmenBreite(rahmenBreite.getSelection());

			einstellungen.setStatusFarben(farbenMap);

			final AnlagenStatus view = (AnlagenStatus) workbench
					.getActiveWorkbenchWindow().getActivePage().findView(
							AnlagenStatus.VIEW_ID);

			if (view != null) {
				view.refreshView();
			}

			PluginEinstellungen.getInstance().saveEinstellungen();
		}
	}

	/**
	 * Testet, ob &Auml;nderungen durchgef&uuml;hrt wurden.
	 * 
	 * @return true, wenn &Auml;nderungen, sonst false
	 */
	private boolean isChanged() {
		final AnlagenStatusEinstellungen einstellungen = PluginEinstellungen
				.getInstance().getEinstellungen();

		if ((einstellungen.isSteuerModuleAusblenden() != steuerModuleInitialAusblenden
				.getSelection())
				|| (einstellungen.isTopDown() != !hierarchieInitial
						.getSelection())
				|| (einstellungen.getLinienFarbe() != ColorUtilities
						.rgbToLong(linienFarbe.getColorValue()))
				|| (einstellungen.getLinienBreite() != linienDicke
						.getSelection())
				|| (einstellungen.getKnotenBackGroundColor() != ColorUtilities
						.rgbToLong(knotenFarbe.getColorValue()))
				|| (einstellungen.getRahmenBreite() != rahmenBreite
						.getSelection())) {
			return true;
		}

		for (AnlagenStatusKnotenStatus stat : AnlagenStatusKnotenStatus
				.values()) {
			// Long long2 = einstellungen.getFarben().get(stat);
			// Long long1 = farbenMap.get(stat);
			if (einstellungen.getStatusFarben().get(stat) != farbenMap
					.get(stat)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * &Uuml;berpr&uuml;ft alle Eingaben.
	 */
	private void checkPage() {

		if (!PluginAnlagenStatus.getDefault().istOnline()) {
			setMessage(resourceBundle
					.getString("EinstellungenOfflineMeldung.label"), WARNING);
			return;
		}
	}

	/**
	 * Label-Provider f&uuml;r Statusfarbenliste.
	 * 
	 * @author BitCtrl Systems GmbH, Gieseler
	 */
	class StatusFarbenLabelProvider implements ILabelProvider {

		/** {@inheritDoc} */
		@Override
		public Image getImage(final Object element) {
			if (element instanceof AnlagenStatusKnotenStatus) {
				final Display display = Display.getDefault();
				final Image image = new Image(display, 16, 16);
				// final Color color = farbenMap.get(element);
				if (farbenMap.get(element) == null) {
					System.err.println("FEHLER");
					return null;
				}
				final Color color = WidgetColorRegistry.getInstance().getColor(
						ColorUtilities.longToRgb(farbenMap.get(element)));
				final GC gc = new GC(image);
				gc.setBackground(color);
				gc.fillRectangle(image.getBounds());
				gc.dispose();
				imagelist.add(image);
				return image;
			}

			return null;
		}

		/** {@inheritDoc} */
		@Override
		public String getText(final Object element) {
			if (element instanceof AnlagenStatusKnotenStatus) {
				return ((AnlagenStatusKnotenStatus) element).getName();
			}
			return null;
		}

		/** {@inheritDoc} */
		@Override
		public void dispose() {
			for (Image i : imagelist) {
				i.dispose();
			}
		}

		/** {@inheritDoc} */
		@Override
		public void addListener(final ILabelProviderListener listener) {
			// TODO Auto-generated method stub

		}

		/** {@inheritDoc} */
		@Override
		public boolean isLabelProperty(final Object element,
				final String property) {
			// TODO Auto-generated method stub
			return false;
		}

		/** {@inheritDoc} */
		@Override
		public void removeListener(final ILabelProviderListener listener) {

		}

	}
}
