/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.figures;

import java.util.ResourceBundle;

import org.eclipse.draw2d.Label;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.graphics.Font;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.ColorUtilities;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenGeraet;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Geraet;

/**
 * Figur eines Anlagenteils.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenGeraetFigure.java 43864 2013-05-14 11:10:42Z hoesel $
 */
public class AnlagenGeraetFigure extends AnlagenFigure {

	/** Das zugeordnete Ger&auml;t. */
	private Geraet geraet;

	/** ResourceBundle. */
	private final ResourceBundle resourceBundle = PluginAnlagenStatus
			.getDefault().getResourceBundle();

	/**
	 * Konstruktor.
	 * 
	 * @param geraet
	 *            zugeordnetes Ger&auml;t
	 */
	public AnlagenGeraetFigure(final Geraet geraet) {
		updateFigureContent(geraet);
	}

	/**
	 * Gibt den Titeltext zur&uuml;ck.
	 * 
	 * @return Titeltext
	 */
	protected String getTitelText() {
		return "Gert";
	}

	/**
	 * Aktualisiert den Inhalt der Figur.
	 * 
	 * @param geraet
	 *            Geraet
	 */
	public void updateFigureContent(final Geraet geraet) {
		this.geraet = geraet;
		getDataPanel().removeAll();
		handleZoomChanged(1.0);

		setBackgroundColor(ColorUtilities.longToColor(PluginEinstellungen
				.getInstance().getEinstellungen().getKnotenBackGroundColor()));

		final Font defaultFont = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getWorkbench().getDisplay()
				.getSystemFont();

		// Titel
		final Label titelLabel = new Label();

		if (geraet != null) {
			titelLabel.setText(AnlagenGeraet.getTypText(geraet));
		} else {
			titelLabel.setText(getTitelText());
		}

		titelLabel.setFont(JFaceResources.getFontRegistry().getBold(
				defaultFont.getFontData()[0].getName()));

		addDataLabel(titelLabel);

		// Bezeichnung/Name
		final Label bezeichnungLabel = new Label();
		if (geraet != null) {
			bezeichnungLabel.setText(AnlagenGeraet.getBezeichnungText(geraet));
		} else {
			bezeichnungLabel.setText("Bezeichnung");
		}

		// bezeichnungLabel.setFont(defaultFont);

		addDataLabel(bezeichnungLabel);

		// Knotennummer
		String kntext = resourceBundle.getString("Knotennummer.label") + " ";

		if (geraet != null) {
			// final BitCtrlDoModel<Geraet> model = geraet.getModel();
			// final Geraet systemObjekt = model.getSystemObjekt();
			geraet.getKdGeraet().getDatum().getKnotenNummer();
			kntext += geraet.getKdGeraet().getDatum().getKnotenNummer();
		}

		final Label knLabel = new Label(kntext);
		// knLabel.setFont(defaultFont);

		addDataLabel(knLabel);
		updateFigure();
	}

	/**
	 * Gibt das zugeordnete Ger&auml;t zur&uuml;ck.
	 * 
	 * @return das zugeordnete Ger&auml;t
	 */
	public Geraet getGeraet() {
		return geraet;
	}
}
