/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.figures;

import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.onlinedaten.OdTlsGloDeFehler;

/**
 * Figure zur Anzeige des Globalen DE-Fehlers.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: DeFehlerFigure.java 32959 2012-02-29 15:05:11Z gieseler $
 */
public class DeFehlerFigure extends OnlineDatenFigure {

	/**
	 * Fehlerstatus.
	 */
	private Label deFehlerStatus;

	/**
	 * Kanalstatus.
	 */
	private Label deKanalStatus;

	/**
	 * Projektierungsstatus.
	 */
	private Label deProjektierungsStatus;

	/**
	 * Hersteller.
	 */
	private Label deHersteller;

	/**
	 * Konstruktor.
	 * 
	 * @param parentpart
	 *            Parent-Edit-Part
	 */
	public DeFehlerFigure(final AnlagenStatusEditPart parentpart) {
		super(parentpart);
		setLabel("Globaler DE-Fehler:");
		setLayoutManager(new GridLayout(2, false));
		setFont(PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getWorkbench().getDisplay().getSystemFont());
		final AttributeGroup atg = DaVVerbindung.getInstanz()
				.getDaVVerbindung().getDataModel().getAttributeGroup(
						OdTlsGloDeFehler.PID);
		final Aspect asp = OdTlsGloDeFehler.Aspekte.TlsAntwort
				.getSystemObject();

		setDatadescription(new DataDescription(atg, asp));
	}

	/** {@inheritDoc} */
	@Override
	public void showStatus(final String status) {
		removeAll();
		add(new Label(status));
		add(new Label());
	}

	/**
	 * Erzeugt die Tabelle der Ausgaben.
	 */
	private void erzeugeTabelle() {
		add(new Label("TLS-DE-Fehler:"));
		deFehlerStatus = new Label();
		add(deFehlerStatus);

		add(new Label("TLS-Kanalstatus:"));
		deKanalStatus = new Label();
		add(deKanalStatus);

		add(new Label("TLS-Projektierungstatus:"));
		deProjektierungsStatus = new Label();
		add(deProjektierungsStatus);

		add(new Label("Hersteller:"));
		deHersteller = new Label();
		add(deHersteller);

	}

	/** {@inheritDoc} */
	@Override
	public void showDaten(final ResultData result) {
		if (deFehlerStatus == null) {
			removeAll();
			erzeugeTabelle();
		}

		final Data data = result.getData();

		deFehlerStatus.setText(data.getItem("DEFehlerStatus").valueToString());
		deKanalStatus.setText(data.getItem("DEKanalStatus").valueToString());
		deProjektierungsStatus.setText(data.getItem("DEProjektierungsStatus")
				.valueToString());
		deHersteller.setText(data.getItem("Hersteller").valueToString());
	}
}
