/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.figures;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Label;
import org.eclipse.swt.graphics.Color;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.AnlagenStatusFunktionsgruppe;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.ColorUtilities;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.konfigurationsdaten.KdDe.Daten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.De;

/**
 * Figur eines Datenendger&auml;tes (DE).
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: DeFigure.java 33160 2012-03-20 13:47:56Z gieseler $
 */
public class DeFigure extends AnlagenFigure {

	/**
	 * Konstruktor.
	 * 
	 * @param de
	 *            {@link De}
	 */
	public DeFigure(final De de) {

		final Label label = new Label();

		if (de != null) {
			setBackgroundColor(getHintergrundFarbe(de));
			final Daten datum = de.getKdDe().getDatum();
			label.setText(datum.getDEKanal().toString());
		} else {
			label.setText("DE");
		}

		addDataLabel(label);
	}

	/**
	 * Bestimmt die Hintergrundfarbe in Abh&auml;hgigkeit vom der
	 * Funktionsgruppe des De.
	 * 
	 * @param systemobjekt
	 *            Systemobjekt
	 * 
	 * @return Hintergrundfarbe f&uuml;r die Figur.
	 * 
	 */
	private Color getHintergrundFarbe(final SystemObjekt systemobjekt) {
		final Long fgFarbe = PluginEinstellungen
				.getInstance()
				.getEinstellungen()
				.getFGFarbe(
						AnlagenStatusFunktionsgruppe.bestimmeFG(systemobjekt));

		if (fgFarbe != null) {
			return ColorUtilities.longToColor(fgFarbe);
		}

		return ColorConstants.gray;

	}

	/** {@inheritDoc} */
	@Override
	public void setScale(final double scale) {

		if (getParent() instanceof EakFigure) {
			return;
		}

		super.setScale(scale);
	}

	/** {@inheritDoc} */
	@Override
	public void handleZoomChanged(final double newZoom) {

		if (getParent() instanceof EakFigure) {
			return;
		}

		super.handleZoomChanged(newZoom);
	}
}
