/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.figures;

import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.Panel;

import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.konfigurationsdaten.KdDe.Daten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.De;

/**
 * Status-Figur f&uuml;r DE-Editparts.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: DeStatusFigure.java 32959 2012-02-29 15:05:11Z gieseler $
 */
public class DeStatusFigure extends StatusFigure {

	/**
	 * Der zugeh&ouml;rige De-Kanal.
	 */
	private final De de;

	/**
	 * Figure globaler DE-Fehler.
	 */
	private final DeFehlerFigure deFehlerPanel;

	/**
	 * Figure erg&auml;nzender DE-Fehler.
	 */
	private final DeErgaenzendeFehlermeldungFigure deErgFehlerPanel;

	/**
	 * Panel f&uuml;r die einzelnen Teile.
	 */
	private final Panel dePanel;

	/**
	 * Konstruktor.
	 * 
	 * @param parentpart
	 *            Parent-Editpart
	 */
	public DeStatusFigure(final AnlagenStatusEditPart parentpart) {
		super(parentpart);

		dePanel = new Panel();
		dePanel.setLayoutManager(new GridLayout(2, false));

		final SystemObjekt so = parentpart.getModel().getSystemObjekt();

		de = (De) so;
		final Daten datum = de.getKdDe().getDatum();

		dePanel.add(new Label("Cluster:"));
		dePanel.add(new Label(datum.getCluster().toString()));
		dePanel.add(new Label("DE-Kanal:"));
		dePanel.add(new Label(datum.getDEKanal().toString()));
		dePanel.add(new Label("EA-Kanal:"));
		dePanel.add(new Label(datum.getEAKanal().toString()));
		dePanel.add(new Label("Umsetzungsmodul:"));
		dePanel.add(new Label(datum.getUmsetzungsModul()));

		dePanel.add(new Label());
		dePanel.add(new Label());

		deFehlerPanel = new DeFehlerFigure(parentpart);
		final GridData gridData = new GridData();
		// gridData.horizontalAlignment = SWT.BEGINNING;
		// gridData.verticalAlignment = GridData.VERTICAL_ALIGN_BEGINNING;
		// gridData.grabExcessHorizontalSpace = true;
		// gridData.grabExcessVerticalSpace = false;
		gridData.horizontalSpan = 2;
		dePanel.getLayoutManager().setConstraint(deFehlerPanel, gridData);
		deFehlerPanel.addPropertyChangeListener(this);

		dePanel.add(deFehlerPanel);

		deErgFehlerPanel = DeErgaenzendeFehlermeldungFigure
				.erzeugeFigure(parentpart);
		if (deErgFehlerPanel != null) {
			final GridData gridData2 = new GridData();
			gridData2.horizontalSpan = 2;
			dePanel.getLayoutManager().setConstraint(deErgFehlerPanel,
					gridData2);
			deErgFehlerPanel.addPropertyChangeListener(this);
			dePanel.add(deErgFehlerPanel);
		}

		addDatenFigure(dePanel);
	}
}
