/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.anlagenstatus.figures;

import java.util.ResourceBundle;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.Panel;

import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Geraet;

/**
 * Figur zur Anzeige von allgemeinen Informationen zu einem Anlagenteil.
 * <p>
 * Angezeigt werden:<br>
 * Name<br>
 * Bezeichnung (wenn konfiguriert)<br>
 * Pid<br>
 * Typ
 * </p>
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: InfoFigure.java 43864 2013-05-14 11:10:42Z hoesel $
 */
public class InfoFigure extends Panel {

	/**
	 * erzeugt eine Instanz der Tooltipfigur.
	 * 
	 * Der Tooltip wird als Panel mit der zwei untereinander angeordneten
	 * Labels, die Name bzw. PID des bergebenen Systemobjekts enthalten,
	 * realisiert.
	 * 
	 * @param systemObjekt
	 *            das Systemobjekt fr den der Tooltip angelegt werden soll
	 */
	public InfoFigure(final SystemObjekt systemObjekt) {
		final ResourceBundle resourceBundle = PluginAnlagenStatus.getDefault()
				.getResourceBundle();

		setBackgroundColor(ColorConstants.tooltipBackground);

		setLayoutManager(new GridLayout(2, false));

		// Name
		add(new Label("Name: "));
		add(new Label(systemObjekt.getName()));

		// Bezeichnung
		if (systemObjekt instanceof Geraet) {
			String text = ((Geraet) systemObjekt).getKdGeraet().getDatum()
					.getBezeichnung();
			if (text == null) {
				text = "";
			}

			add(new Label("Bezeichnung: "));
			add(new Label(text));
		}

		// Pid
		add(new Label("Pid: "));
		add(new Label(systemObjekt.getPid()));

		// Typ
		add(new Label(resourceBundle.getString("Typ.label") + " "));
		add(new Label(systemObjekt.getTyp().getName() + " ("
				+ systemObjekt.getTyp().getPid() + ")"));

	}
}
