/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.anlagenstatus.figures;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.GridLayout;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LabeledContainer;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;

/**
 * Figure zur Anzeige der Versorgungsinformationen.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: VersorgungFigure.java 43864 2013-05-14 11:10:42Z hoesel $
 */
public class VersorgungFigure extends DatenFenster {

	/** Modell. */
	private final AnlagenStatusKnoten model;

	/** Liste der bereits angezeigten DataDescriptions. */
	private final List<DataDescription> descriptionListe = new ArrayList<DataDescription>();

	/**
	 * Konstruktor.
	 * 
	 * @param parentpart
	 *            Parent-Edit-Part.
	 */
	public VersorgungFigure(final AnlagenStatusEditPart parentpart) {
		super(parentpart);

		model = parentpart.getKnoten();

		final String header = PluginAnlagenStatus.getDefault()
				.getResourceBundle().getString("VersorgungsdatenFenster.label")
				+ " " + model.getSystemObjekt().getName();
		setLabel(header);

		getLayoutManager().layout(this);
		aktualisiereLayout();
	}

	/**
	 * F&uuml;gt Versorgungsdaten zur Anzeige in der Figur hinzu.
	 * 
	 * @param atg
	 *            Attributgruppe
	 * @param asp
	 *            Aspekt
	 */
	public void addVersorgungsDaten(final AttributeGroup atg, final Aspect asp) {
		final Data configurationData = model.getSystemObject()
				.getConfigurationData(atg, asp);
		final DataDescription dd = new DataDescription(atg, asp);
		erzeugeDatenFigure(dd, configurationData);
	}

	/**
	 * Erzeugt eine Figur zur Anzeige von Daten.
	 * 
	 * @param desc
	 *            DataDescription
	 * @param data
	 *            Daten
	 */
	private void erzeugeDatenFigure(final DataDescription desc, final Data data) {
		final LabeledContainer datenFigure = new LabeledContainer();

		datenFigure.setLabel(desc.getAttributeGroup().getName() + "("
				+ desc.getAttributeGroup().getPid() + ")");
		datenFigure.setLayoutManager(new GridLayout(2, false));
		datenFigure.setFont(PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getWorkbench().getDisplay()
				.getSystemFont());

		descriptionListe.add(desc);

		if (data == null) {
			datenFigure.add(new Label("keine Daten"));
			datenFigure.add(new Label());
		} else {
			for (Attribute a : desc.getAttributeGroup().getAttributes()) {
				datenFigure.add(new Label(a.getName() + ":"));
				datenFigure.add(new Label(data.getTextValue(a.getName())
						.getText()));
			}
		}

		addDatenFigure(datenFigure);
	}

	/**
	 * Bestimmt, ob eine ATG/ASP-Kombination bereits im Versorgungsfenster
	 * angezeigt wird.
	 * 
	 * @param atg
	 *            Attributgruppe
	 * @param asp
	 *            Aspekt
	 * @return true, wenn bereits angezeigt, sonst false
	 */
	public boolean isInVersorgung(final AttributeGroup atg, final Aspect asp) {
		final DataDescription dd = new DataDescription(atg, asp);
		return descriptionListe.contains(dd);
	}
}
