/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.modell;

import java.util.ResourceBundle;

import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.De;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Eak;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Geraet;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Kri;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Kri2B;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.SteuerModul;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Uz;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Viz;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Vrz;

/**
 * Utility-Klasse.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenGeraet.java 43864 2013-05-14 11:10:42Z hoesel $
 */
public final class AnlagenGeraet {

	/**
	 * Konstruktor.
	 */
	private AnlagenGeraet() {
		//
	}

	/**
	 * Gibt den Typtext des Ger&auml;tes zur&uuml;ck.
	 * 
	 * @param systemobjekt
	 *            SystemObjekt
	 * 
	 * @return Typtext des Ger&auml;tes
	 */
	public static String getTypText(final SystemObjekt systemobjekt) {
		final ResourceBundle resourceBundle = PluginAnlagenStatus.getDefault()
				.getResourceBundle();

		String text = resourceBundle.getString("GeraetUnbekannt.label");
		if (systemobjekt instanceof Vrz) {
			text = resourceBundle.getString("GeraetVRZ.label");
		} else if (systemobjekt instanceof Viz) {
			text = resourceBundle.getString("GeraetVIZ.label");
		} else if (systemobjekt instanceof Uz) {
			text = resourceBundle.getString("GeraetUZ.label");
		} else if (systemobjekt instanceof Kri) {
			text = resourceBundle.getString("GeraetKRI.label");
		} else if (systemobjekt instanceof Kri2B) {
			text = resourceBundle.getString("GeraetKRI2B.label");
		} else if (systemobjekt instanceof SteuerModul) {
			text = resourceBundle.getString("GeraetSteuermodul.label");
		} else if (systemobjekt instanceof Eak) {
			text = resourceBundle.getString("GeraetEAK.label");
		} else if (systemobjekt instanceof De) {
			text = resourceBundle.getString("GeraetDE.label");
		}
		return text;
	}

	/**
	 * Gibt die Bezeichnung des Ger&auml;tes zur&uuml;ck.
	 * 
	 * @param systemobjekt
	 *            SystemObjekt
	 * 
	 * @return Bezeichnung des Ger&auml;tes
	 */
	public static String getBezeichnungText(final SystemObjekt systemobjekt) {
		String text = "";

		if (systemobjekt instanceof Geraet) {
			text = ((Geraet) systemobjekt).getKdGeraet().getDatum()
					.getBezeichnung();
		}

		if (text == null || text.length() == 0) {
			text = systemobjekt.getName();
		}
		return text;
	}
}
