/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.modell;

import org.eclipse.draw2d.AbstractConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;

/**
 * Connection-Anchor des Anlagenstatus-View.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusConnectionAnchor.java 33800 2012-04-26 14:40:48Z gieseler $
 */
public class AnlagenStatusConnectionAnchor extends AbstractConnectionAnchor {

	/** Name. */
	private String name = "Anlagenstatus-Anchor";

	/** . */
	public boolean leftToRight = true;

	/** . */
	public boolean topDown = true;

	/** Horizintaler Offset. */
	public int offsetH;

	/** Vertikaler Offset. */
	public int offsetV;

	/**
	 * Konstruktor.
	 * 
	 * @param iOwner
	 *            Figur
	 */
	public AnlagenStatusConnectionAnchor(final IFigure iOwner) {
		super(iOwner);
	}

	/**
	 * Gibt den horizontalen Offset zur&uuml;ck.
	 * 
	 * @return horizontaler Offset
	 */
	public int getOffsetH() {
		return offsetH;
	}

	/**
	 * Setzt den horizontalen Offset.
	 * 
	 * @param offsetH
	 *            horizontaler Offset
	 */
	public void setOffsetH(final int offsetH) {
		this.offsetH = offsetH;
	}

	/**
	 * Gibt den vertikalen Offset zur&uuml;ck.
	 * 
	 * @return Offset
	 */
	public int getOffsetV() {
		return offsetV;
	}

	/**
	 * Setzt den vertikalen Offset.
	 * 
	 * @param offsetV
	 *            vertikaler Offset
	 */
	public void setOffsetV(final int offsetV) {
		this.offsetV = offsetV;
	}

	/** {@inheritDoc} */
	public Point getLocation(final Point iReference) {
		Rectangle r = getOwner().getBounds();
		int x, y;
		if (topDown) {
			y = r.y + offsetV;
		} else {
			y = r.y + r.height - offsetV;
		}

		if (leftToRight) {
			x = r.x + offsetH;
		} else {
			x = r.x + r.width - offsetH;
		}

		Point p = new Point(x, y);
		getOwner().translateToAbsolute(p);

		return p;
	}

	/**
	 * Setzt den Namen.
	 * 
	 * @param aName
	 *            name
	 */
	public void setName(final String aName) {
		name = aName;
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		return name;
	}
}
