/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.parts;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartFactory;

import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusDeKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusEakKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusGeraetKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusModell;
import de.bsvrz.buv.plugin.anlagenstatus.modell.SteuerModulKnoten;
import de.bsvrz.buv.plugin.dobj.model.BitCtrlDoModel;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.De;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Eak;

/**
 * Edit-Part-Factory.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusEditPartFactory.java 33710 2012-04-20 11:06:38Z gieseler $
 */
public class AnlagenStatusEditPartFactory implements EditPartFactory {

	/** {@inheritDoc} */
	@Override
	public EditPart createEditPart(final EditPart context, final Object model) {
		// System.out.println("AnlagenStatusEditPartFactory.createEditPart("
		// + context + "," + model + ")");

		EditPart editPart = null;

		// System.out.println("createEditPart model: " + model);
		if (model instanceof AnlagenStatusModell) {
			editPart = new AnlagenStatusTopEditPart();
		} else if (model instanceof SteuerModulKnoten) {
			editPart = new AnlagenStatusSteuerModulEditPart();
		} else if (model instanceof AnlagenStatusDeKnoten) {
			editPart = new AnlagenStatusDeEditPart();
		} else if (model instanceof AnlagenStatusEakKnoten) {
			editPart = new EakEditPart();
		} else if (model instanceof AnlagenStatusGeraetKnoten) {
			editPart = new AnlagenStatusGeraetEditPart();
		} else if (model instanceof BitCtrlDoModel<?>) {
			final BitCtrlDoModel<?> bmodel = (BitCtrlDoModel<?>) model;
			final SystemObjekt systemObjekt = bmodel.getSystemObjekt();
			// System.out.println("createEditPart Objekt: " + systemObjekt);
			if (systemObjekt instanceof Eak) {
				editPart = new AnlagenStatusEakEditPart();
			} else if (systemObjekt instanceof De) {
				editPart = new AnlagenStatusDeEditPart();
			}
		} else if (model instanceof AnlagenStatusKnoten) {
			editPart = new AnlagenStatusGeraetEditPart();
		}

		if (editPart != null) {
			editPart.setModel(model);
		}
		return editPart;
	}

}
