/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.parts;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.draw2d.AutomaticRouter;
import org.eclipse.draw2d.BendpointConnectionRouter;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.FanRouter;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.FreeformLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusModell;
import de.bsvrz.buv.plugin.anlagenstatus.modell.BusRouter;

/**
 * Top-Edit-Part f&uuml;r die Ansicht Anlagenstatus.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * @version $Id: AnlagenStatusTopEditPart.java 33160 2012-03-20 13:47:56Z gieseler $
 */
public class AnlagenStatusTopEditPart extends AbstractGraphicalEditPart {

	/**
	 * Margin.
	 */
	private static final int MARGIN = 10;

	/**
	 * Level-Map.
	 */
	private final Map<Integer, List<AnlagenStatusEditPart>> levelMap = new HashMap<Integer, List<AnlagenStatusEditPart>>();

	/** Singleton instance of MarqueeDragTracker. */
	// static DragTracker m_dragTracker;

	/**
	 * If the following static variable is set to 'true', then Manhattan routing
	 * is used, otherwise connections with bendpoints is used.
	 **/
	// static boolean s_isManhattanRouting;

	/**
	 * Gibt die Level-Map zur&uuml;ck.
	 * 
	 * @return Level-Map
	 */
	public Map<Integer, List<AnlagenStatusEditPart>> getLevelMap() {
		return levelMap;
	}

	/** {@inheritDoc} */
	@Override
	protected IFigure createFigure() {
		final Figure f = new FreeformLayer();
		f.setLayoutManager(new FreeformLayout());
		f.setBorder(new MarginBorder(MARGIN));

		// Don't know why, but if you don't setOpaque(true), you cannot move by
		// drag&drop!
		// f.setOpaque(true);
		return f;
	}

	/** {@inheritDoc} */
	@Override
	protected void createEditPolicies() {
		// installEditPolicy(EditPolicy.LAYOUT_ROLE, new
		// GefTestXYLayoutEditPolicy());
	}

	/** {@inheritDoc} */
	@Override
	protected List<?> getModelChildren() {
		final List<AnlagenStatusKnoten> children = ((AnlagenStatusModell) getModel())
				.getViewNodes();

		return children;
	}

	/** {@inheritDoc} */
	@Override
	public void activate() {
		if (!isActive()) {
			super.activate();
		}
	}

	// /** {@inheritDoc} */
	// @Override
	// public DragTracker getDragTracker(final Request req) {
	// if (m_dragTracker == null) {
	// m_dragTracker = new MarqueeDragTracker();
	// }
	// return m_dragTracker;
	// }

	/** {@inheritDoc} */
	@Override
	protected void refreshVisuals() {
		final ConnectionLayer cLayer = (ConnectionLayer) getLayer(LayerConstants.CONNECTION_LAYER);

		// It seems that Manhanttan routing cannot be used with bendpoints...
		if (((AnlagenStatusModell) getModel()).isManhattanrouting()) {
			// cLayer.setConnectionRouter(new ManhattanConnectionRouter());
			cLayer.setConnectionRouter(new BusRouter());
		} else {
			final AutomaticRouter router = new FanRouter();
			router.setNextRouter(new BendpointConnectionRouter());
			cLayer.setConnectionRouter(router);
		}

		final List<?> children = getChildren();

		for (int i = 0; i < children.size(); i++) {
			// ((EditPart) children.get(i)).refresh();
			((AnlagenStatusEditPart) children.get(i)).refreshVisuals();
		}
	}

	/**
	 * Gibt den Vorg&auml;nger in der Hierarchie zur&uuml;ck.
	 * 
	 * @param part
	 *            Part
	 * @return Vorg&auml;nger in der Hierarchie
	 */
	public AnlagenStatusEditPart getVorgaenger(final AnlagenStatusEditPart part) {
		final int level = ((AnlagenStatusKnoten) part.getModel()).getlevel();
		final List<AnlagenStatusEditPart> list = levelMap.get(level);
		if (list == null || list.isEmpty()) {
			return null;
		}

		final int indexOf = list.indexOf(part);

		if (indexOf <= 0) {
			return list.get(list.size() - 1);
		}

		return list.get(indexOf - 1);
	}

	/**
	 * F&uuml;gt einen Editpart hinzu.
	 * 
	 * @param level
	 *            Hierarchie-Level
	 * @param part
	 *            Part
	 */
	public void addEditPart(final int level, final AnlagenStatusEditPart part) {
		List<AnlagenStatusEditPart> list = levelMap.get(level);
		if (list == null) {
			list = new ArrayList<AnlagenStatusEditPart>();
			list.add(part);
			levelMap.put(level, list);
		} else {
			if (!list.contains(part)) {
				list.add(part);
			}
		}
	}

	/**
	 * Gibt die Y-Position f&uuml;r ein Hierarchie-Level zur&uuml;ck.
	 * 
	 * @param level
	 *            Hierarchie-Level
	 * @return Y-Position f&uuml;r Hierarchie-Level
	 */
	public int getTopDownY(final int level) {
		if (level == 0) {
			return 5;
		}

		final AnlagenStatusModell model = (AnlagenStatusModell) getModel();

		int v = model.getVerbindungenMax();

		v *= PluginEinstellungen.getInstance().getEinstellungen()
				.getLinienBreite()
				* level;

		final int downy = Math.max(level * 200, v + 50);

		return downy + 5;
	}

	/**
	 * Gibt die X-Position f&uuml;r ein Hierarchie-Level zur&uuml;ck.
	 * 
	 * @param level
	 *            Hierarchie-Level
	 * @return X-Position f&uuml;r Hierarchie-Level
	 */
	public int getLeftRightX(final int level) {

		if (level == 0) {
			return 5;
		}

		final AnlagenStatusModell model = (AnlagenStatusModell) getModel();

		int v = model.getVerbindungenMax();

		v *= PluginEinstellungen.getInstance().getEinstellungen()
				.getLinienBreite()
				* level;

		final int rightx = Math.max(level * 350, v + 50);

		return rightx + 5;

		// return level * 250 + 5;
	}

	// public ConnectionLayer getConnectionLayer() {
	// return (ConnectionLayer) getLayer(LayerConstants.CONNECTION_LAYER);
	// }

	// /**
	// * Berechnet das gesamte Bild neu.
	// */
	// public void bildNeuBerechnen() {
	// final List<?> children = getChildren();
	//
	// for (int i = 0; i < children.size(); i++) {
	// // ((EditPart) children.get(i)).refresh();
	// ((AnlagenStatusEditPart) children.get(i)).bildNeuBerechnen();
	// }
	//
	// refresh();
	// }
}
