/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.anlagenstatus.parts;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.GridData;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPartListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.ColorUtilities;
import de.bsvrz.buv.plugin.anlagenstatus.einstellungen.PluginEinstellungen;
import de.bsvrz.buv.plugin.anlagenstatus.export.HtmlTools;
import de.bsvrz.buv.plugin.anlagenstatus.figures.AnlagenFigure;
import de.bsvrz.buv.plugin.anlagenstatus.figures.EakFigure;
import de.bsvrz.buv.plugin.anlagenstatus.figures.GeraetStatusFigure;
import de.bsvrz.buv.plugin.anlagenstatus.figures.StatusFigure;
import de.bsvrz.buv.plugin.anlagenstatus.internal.AnlagenStatusKnotenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.internal.PluginAnlagenStatus;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusConnection;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusModell;
import de.bsvrz.buv.plugin.anlagenstatus.tooltips.AnlagenStatusToolTipFigure;
import de.bsvrz.buv.plugin.dobj.model.BitCtrlDoModel;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.buv.plugin.dobj.model.DobjPackage;
import de.bsvrz.buv.plugin.dobj.util.DoTypenUtil;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.konfigurationsdaten.KdEak.Daten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Eak;

/**
 * EAK-Editpart.
 * 
 * @author BitCtrl Systems GmbH, Steffen Gieseler
 * 
 * @version $Id: EakEditPart.java 43864 2013-05-14 11:10:42Z hoesel $
 */
public class EakEditPart extends AnlagenStatusEditPart implements
		EditPartListener {

	/**
	 * ID des EditParts.
	 */
	public static final String PART_ID = EakEditPart.class.getName();

	/**
	 * Kinder.
	 */
	private final List<AnlagenStatusEditPart> childParts = new ArrayList<AnlagenStatusEditPart>();

	/**
	 * Die Figure des EAK selbst.
	 */
	private AnlagenFigure eakFigure;

	/** {@inheritDoc} */
	@Override
	protected IFigure createFigure() {
		addEditPartListener(this);
		return new EakFigure();
	}

	/** {@inheritDoc} */
	@Override
	protected List<?> getModelChildren() {
		final AnlagenStatusModell anlagenModell = PluginAnlagenStatus
				.getDefault().getAnlagenModell();

		final AnlagenStatusKnoten knoten = anlagenModell.getKnoten(getModel()
				.getSystemObjekt());

		if (knoten == null || knoten.getKinder().size() == 0) {
			return Collections.EMPTY_LIST;
		}

		final ConfiguratedDoTyp doTyp = DoTypenUtil
				.getConfiguratedDoTyp(DeEditPart.PART_ID);

		Assert.isNotNull(doTyp, "Der DO-Typ '" + DeEditPart.PART_ID
				+ "' konnte nicht gefunden werden!");

		// Liste als BitCtrlDoModel<SystemObjekt> zusammenstellen
		final List<BitCtrlDoModel<SystemObjekt>> kliste = new ArrayList<BitCtrlDoModel<SystemObjekt>>();

		for (final AnlagenStatusKnoten k : knoten.getKinder()) {
			// @SuppressWarnings("unchecked")
			// final BitCtrlDoModel<SystemObjekt> bitctrlDoModel =
			// (BitCtrlDoModel<SystemObjekt>) doTyp
			// .createModel();
			//
			// bitctrlDoModel.setSystemObjekt(k.getSystemObjekt());

			k.setDoTyp(doTyp);

			// kliste.add(bitctrlDoModel);
			kliste.add(k);
		}

		return kliste;
	}

	/** {@inheritDoc} */
	@Override
	public void childAdded(final EditPart child, final int index) {
		childParts.add((AnlagenStatusEditPart) child);
		// refresh();
		erzeugeFigureInhalt();
	}

	/**
	 * Positioniert die Kinder.
	 */
	private void positioniereKinder() {
		final Dimension figureSize = getFigure().getSize();

		for (AnlagenStatusEditPart part : childParts) {
			final Point childLocation = new Point(figureSize.width, 0);
			final Dimension childSize = part.getFigure().getSize();

			figureSize.width += part.getFigure().getSize().width;

			getFigure().setSize(figureSize);

			getFigure().add(part.getFigure());
			getFigure().getLayoutManager().setConstraint(
					part.getFigure(),
					new Rectangle(childLocation.x, childLocation.y,
							childSize.width, childSize.height));

			part.getModel().setLocation(childLocation);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void notifyChanged(final Notification notification) {
		super.notifyChanged(notification);

		final Object notifier = notification.getNotifier();
		final int type = notification.getEventType();

		if (notifier instanceof BitCtrlDoModel<?>) {
			final int featureID = notification
					.getFeatureID(BitCtrlDoModel.class);
			switch (type) {
			case Notification.SET:
				switch (featureID) {
				case DobjPackage.BIT_CTRL_DO_MODEL__SYSTEM_OBJEKT:
					childParts.clear();
					refreshChildren();
					break;
				default:
					break;
				}
				break;
			default:
				break;
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void partActivated(final EditPart editpart) {
	}

	/** {@inheritDoc} */
	@Override
	public void partDeactivated(final EditPart editpart) {
	}

	/** {@inheritDoc} */
	@Override
	public void removingChild(final EditPart child, final int index) {
	}

	/** {@inheritDoc} */
	@Override
	public void selectedStateChanged(final EditPart editpart) {
	}

	/** {@inheritDoc} */
	@Override
	protected void erzeugeFigureInhalt() {

		((Figure) getFigure()).removeAll();

		eakFigure = new AnlagenFigure();

		eakFigure.setBackgroundColor(ColorUtilities
				.longToColor(PluginEinstellungen.getInstance()
						.getEinstellungen().getKnotenBackGroundColor()));

		final Label mLabel = new Label();
		mLabel.setLabelAlignment(PositionConstants.CENTER);
		mLabel.setLabelAlignment(PositionConstants.MIDDLE);
		final Font font = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getWorkbench().getDisplay().getSystemFont();
		mLabel.setFont(font);

		if (getModel().getSystemObjekt() != null) {
			final Daten datum = ((Eak) getModel().getSystemObjekt()).getKdEak()
					.getDatum();

			mLabel.setText("EAK(" + datum.getOSI2Adresse().toString() + ")");
		} else {
			mLabel.setText("EAK");
		}

		final GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.CENTER;
		gridData.verticalAlignment = GridData.CENTER;
		gridData.grabExcessHorizontalSpace = true;

		eakFigure.addToFigure(mLabel, gridData);

		getFigure().setToolTip(
				AnlagenStatusToolTipFigure.getTooltipFigure(this));

		final Dimension cSize = eakFigure.getPreferredSize();

		getFigure().add(eakFigure);
		getFigure().getLayoutManager().setConstraint(eakFigure,
				new Rectangle(0, 0, cSize.width, cSize.height));
		getFigure().setSize(cSize);

		positioniereKinder();

		((EakFigure) getFigure()).setDimension(null);
		((EakFigure) getFigure()).updateFigure();
	}

	/** {@inheritDoc} */
	@Override
	protected void updateBorder() {
		AnlagenStatusKnotenStatus anzeigeStatus = getKnotenStatus();

		if (!isPartOnline()) {
			anzeigeStatus = AnlagenStatusKnotenStatus.UNBEKANNT;
		}

		eakFigure.setBorder(new LineBorder(ColorUtilities
				.longToColor(PluginEinstellungen.getInstance()
						.getEinstellungen()
						.getKnotenStatusBorderColor(anzeigeStatus)),
				PluginEinstellungen.getInstance().getEinstellungen()
						.getRahmenBreite()));
	}

	/** {@inheritDoc} */
	@Override
	public String getHtmlText(final int col, final int cols, final boolean first) {
		final StringBuffer sb = new StringBuffer();
		final SystemObjekt systemobjekt = getModel().getSystemObjekt();

		sb.append("<table border=\"5\" bordercolor=\"" + getHtmlBorderColor()
				+ "\" rules=\"none\" >\n");

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(null,
				HtmlTools.erzeugeHtmlTabelleSpalte(null, getHtmlBasisInfo())));

		sb.append("<tr><td><table border=\"0\" rules=\"none\" >\n");

		// Bezeichnung/Name
		final String bez = ((Eak) systemobjekt).getKdEak().getDatum()
				.getBezeichnung();
		if (bez != null && bez.length() == 0) {
			sb.append(HtmlTools.erzeugeHtmlTabelleZeile(null,
					HtmlTools.erzeugeHtmlTabelleSpalte(null, "Bezeichnung:")
							+ HtmlTools.erzeugeHtmlTabelleSpalte(null, bez)));
		}

		sb.append(HtmlTools.erzeugeHtmlTabelleZeile(
				null,
				HtmlTools.erzeugeHtmlTabelleSpalte(null, "Status:")
						+ HtmlTools.erzeugeHtmlTabelleSpalte(null,
								getKnotenStatus().getName())));
		sb.append("</table></td></tr>\n");

		sb.append("</table>\n");

		final StringBuffer sba = new StringBuffer();
		sba.append(HtmlTools.erzeugeHtmlTabelleSpalte(null, sb.toString()));

		for (Object o : getChildren()) {
			final AnlagenStatusEditPart ep = (AnlagenStatusEditPart) o;
			sba.append(HtmlTools.erzeugeHtmlTabelleSpalte(null,
					ep.getHtmlText(col, cols, first)));
		}

		return HtmlTools.erzeugeHtmlElement("table",
				"border=\"0\" cellpadding=\"0\" rules=\"none\"",
				HtmlTools.erzeugeHtmlTabelleZeile(null, sba.toString()));
	}

	/** {@inheritDoc} */
	@Override
	protected StatusFigure erzeugeStatusFigure() {
		return new GeraetStatusFigure(this);
	}

	/** {@inheritDoc} */
	@Override
	protected List<AnlagenStatusConnection> getModelSourceConnections() {
		return Collections.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	protected List<AnlagenStatusConnection> getModelTargetConnections() {
		return Collections.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	void setPositionUndGroesse() {
	}
}
