/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.views;

import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditPartViewer;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;

import de.bsvrz.buv.plugin.anlagenstatus.actions.KnotenAusblendenAktion;
import de.bsvrz.buv.plugin.anlagenstatus.actions.KnotenUntergeordneteElementeAusblendenAktion;
import de.bsvrz.buv.plugin.anlagenstatus.actions.KnotenUntergeordneteElementeEinblendenAktion;
import de.bsvrz.buv.plugin.anlagenstatus.actions.StatusAnzeigenAktion;
import de.bsvrz.buv.plugin.anlagenstatus.actions.StatusSchliessenAktion;
import de.bsvrz.buv.plugin.anlagenstatus.actions.VersorgungSchliessenAktion;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusKnoten;
import de.bsvrz.buv.plugin.anlagenstatus.modell.AnlagenStatusModell;
import de.bsvrz.buv.plugin.anlagenstatus.parts.AnlagenStatusEditPart;
import de.bsvrz.buv.plugin.dobj.model.BitCtrlDoModel;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmkextlsglobal.objekte.Geraet;

/**
 * Kontext-Men&uuml; f&uuml;r Anlagenstatus.
 * 
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: $
 */
public class AnlagenStatusContextMenu extends ContextMenuProvider {

	/**
	 * Konstruktor.
	 * 
	 * @param viewer
	 *            Viewer
	 */
	public AnlagenStatusContextMenu(final EditPartViewer viewer) {
		super(viewer);
		// setActionRegistry(actionRegistry);
		setRemoveAllWhenShown(true);
	}

	/** {@inheritDoc} */
	@Override
	public void buildContextMenu(final IMenuManager menu) {
		// GEFActionConstants.addStandardActionGroups(menu);

		final ISelection selection = getViewer().getSelection();

		if (!selection.isEmpty() && selection instanceof StructuredSelection) {
			final StructuredSelection ssel = (StructuredSelection) selection;
			if (ssel.getFirstElement() instanceof AnlagenStatusEditPart) {
				final AnlagenStatusEditPart aep = (AnlagenStatusEditPart) ssel
						.getFirstElement();
				final BitCtrlDoModel<SystemObjekt> model = aep.getModel();

				if (aep.getStatusFigure() != null) {
					menu.add(new StatusSchliessenAktion(aep));
				} else {
					menu.add(new StatusAnzeigenAktion(aep));
				}
				menu.add(new Separator());

				if (aep.getVersorgungFigure() != null) {
					menu.add(new VersorgungSchliessenAktion(aep));
					menu.add(new Separator());
				}

				if (model.getSystemObjekt() instanceof Geraet) {
					if (model instanceof AnlagenStatusKnoten) {
						final AnlagenStatusKnoten modell = (AnlagenStatusKnoten) aep
								.getModel();

						if (modell.getlevel() > AnlagenStatusModell.VRZ_VIZ_LEVEL) {
							menu.add(new KnotenAusblendenAktion(aep));
						}

						if (modell.hasUnsichtbareKinder()) {
							menu.add(new KnotenUntergeordneteElementeEinblendenAktion(
									aep));
						}
						if (modell.hasSichtbareKinder()) {
							menu.add(new KnotenUntergeordneteElementeAusblendenAktion(
									aep));
						}
					}
				}

				menu.add(new Separator());

				menu.add(aep.erzeugeVersorgungsDatenMenu());
			}
		}
	}

}
